from django.test import TestCase, Client
from json import loads
from os import path, getcwd
from .models import Speech
from .templatetags.gTTS import say, temp_path
from .cache import remove_cache

class TranslationStorage_TestCase(TestCase):
    """ requires makemigration and migrate before testing """
    language = 'en'
    text = 'something to say'

    def test_speech_stored_and_returned(self):
        resp = say(
            self.language,
            self.text).split('/')[-1:][0]
        self.assertEquals(
            Speech.objects.get(
                language=self.language,
                text=self.text).file_name
                , resp)

    def test_dynamic_route(self):
        with open('gTTS' + loads(
            Client().get('/gtts/%s/%s' %(
                self.language, self.text
            )).content
        )['mp3'], 'rb') as file:
            resp = file.read()
        with open('gTTS' + say(
            self.language, self.text), 'rb') as file:
            resp2 = file.read()
        self.assertEqual(
            resp, resp2)
    
    def test_dynamic_auth_route(self):
        resp = Client().get('/gtts_auth/%s/%s' %(
            self.language, self.text
        )).status_code
        self.assertEqual(resp, 302)

    def test_remove_cache(self):
        self.assertTrue(
            path.isdir(temp_path)
        )
        remove_cache()
        self.assertFalse(
            path.isdir(temp_path)
        )
    
    def test_say_false_input(self):
        try:
            say(False, False)
        except Exception as e:
            self.assertEquals(
                type(e),
                TypeError
            )


from django import template
from django.conf import settings
from django.contrib.staticfiles.templatetags.staticfiles import static
from gtts import gTTS
from os import path, makedirs, remove
from datetime import datetime
from sys import version_info
from ..models import Speech

cur_dir = path.join(path.dirname(path.abspath(__file__)), '..')
dir_name = 'gTTS'
temp_path = path.join(
    cur_dir, 
    path.join(
        getattr(settings, 'STATIC_URL', ' ')[1:], 
        dir_name
    )
)

register = template.Library()

@register.simple_tag
def say(
    language='en-us', 
    text='Flask says Hi!'):
    for h, a in {'language': language, 'text': text}.items():
        if not isinstance(a, str):  # check if receiving a string
            raise(TypeError("gTTS.say(%s) takes string" % h))
    try:
        ext_file = Speech.objects.get(text=text, language=language)
        if not isfile(path.join(temp_path, ext_file.file_name)):
            for file in Speech.objects.filter(
                text=text, language=language).all():
                file.delete()
            ext_file = None
    except Exception:
        ext_file = None
    if not path.isdir(temp_path):  # creating temporary directory
        makedirs(temp_path) if version_info.major == 2 else makedirs(
            # makedirs in py2 missing exist_ok
            temp_path, exist_ok=True
        )
    if ext_file is None:
        s = gTTS(text) if language == 'skip' else gTTS(
            text,
            language)
        while True:  # making sure audio file name is truly unique
            fname = str(
                datetime.utcnow()
                ).replace('.', ''
                ).replace('-', ''
                ).replace(' ', ''
                ).replace(':', '') + '.mp3'
            abp_fname = path.join(temp_path, fname)
            if not path.isfile(abp_fname):
                break
        Speech(text=text,
        language=language,
        file_name=fname).save()
        s.save(abp_fname)
    else:
        fname = ext_file.file_name
    return static('/'.join([dir_name, fname]))


from shutil import rmtree
from os import path
from .templatetags.gTTS import temp_path
from .models import Speech


def remove_cache():
    """ 
    remove cache folder and Speech modal records 
    """
    if path.isdir(temp_path):
        rmtree(temp_path)
    Speech.objects.all().delete()


from django.shortcuts import render
from django.http import JsonResponse
from django.contrib.auth.decorators import login_required
from .templatetags.gTTS import say
from .cache import remove_cache


def gTTs(r, language='en', text='say it'):
    """ 
    To return json dynamic TTS .mp3 link response :
    {mp3 : 'mp3 static link ...'}
    @param: language language to speak in (default: 'en')
    @param: text text to speak out (default: 'say it')
    """
    return JsonResponse(
        {'mp3': 
        say(
            language=language,
            text=text
        ).replace('%5C', '/')}
    )


@login_required
def gTTs_auth(r, language='en', text='say it'):
    """ 
    To return json dynamic TTS .mp3 link response only for
    authenticated users: {mp3 : 'mp3 static link ...'}
    @param: language language to speak in (default: 'en')
    @param: text text to speak out (default: 'say it')
    """
    return gTTs(r, language, text)


"""
Django-Gtts
-------------
A Django app to add gTTS google text-to-speech to the template 
with ability to cache text-to-speech .mp3 files generated.
"""
from setuptools import setup, find_packages


setup(
    name='Django-Gtts',
    version='0.2',
    packages=find_packages(),
    url='https://github.com/mrf345/django_gtts/',
    download_url='https://github.com/mrf345/django_gtts/archive/0.2.tar.gz',
    license='MIT',
    author='Mohamed Feddad',
    author_email='mrf345@gmail.com',
    description='gTTS google text-to-speech django app',
    long_description=__doc__,
    include_package_data=True,
    platforms='any',
    install_requires=[
        'django',
        'gtts'
    ],
    keywords=['django', 'extension', 'google', 'gtts', 'text-to-speech'],
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ]
)


# Generated by Django 2.0.6 on 2018-08-06 06:59

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Speech',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('text', models.TextField(max_length=2000)),
                ('language', models.TextField(max_length=50)),
                ('file_name', models.TextField(max_length=1000)),
            ],
        ),
    ]
